#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# Copyright (c) 2023-2024, AllWorldIT
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Port knocking functions for the Smart SSH Client."""

import socket
import sys

from .exceptions import SmartSSHClientError

__all__ = ["port_knock"]


def port_knock(family: int, host: str, ports: str) -> None:
    """Perform a port knocking sequence."""

    # Loop with port sequence
    for port in ports.split(","):
        print(f"PROXY INFO: Smart SSH Client - Port knocking host '{host}' on port '{port}'", file=sys.stderr)  # noqa: T201
        try:
            with socket.socket(family, socket.SOCK_STREAM) as knock_sock:
                knock_sock.settimeout(2)
                knock_sock.connect((host, int(port)))
        except TimeoutError:
            pass  # Expected timeout for port knocking
        except OSError as e:
            raise SmartSSHClientError(f"Error port knocking host '{host}' on port '{port}': {e}") from e

    print("PROXY INFO: Smart SSH Client - Port knocking complete", file=sys.stderr)  # noqa: T201
